import { Box } from '../../../classes/box.js';
import { Scene } from '../../../classes/scene.js';
import { Expert } from '../../../classes/expert.js';
import { Bench } from '../../../classes/bench.js';
import { delay } from '../../../util.js';
import { Forum } from '../../../classes/forum.js';
import { PostContent } from '../../../classes/post-content.js';
import params from '../../../params.js';

export class ForumTest {
  constructor(options) {
    this.scene = null;
    this.forum = null;
    this.bench = null;
    this.experts = null;
    this.posts = null;
    this.options = {
      defaultDelayMs: 1,
      poolDurationMs: 50,
      ...options,
    };
  }

  async newExpert() {
    const index = this.experts.length;
    const name = `Expert${index + 1}`;
    const expert = await new Expert(name, this.scene).initialize();
    this.experts.push(expert);
    // expert.addValue('rep', () => this.bench.reputation.valueOwnedBy(expert.reputationPublicKey));
    return expert;
  }

  async addPost(author, fee, citations = []) {
    const postIndex = this.posts.length;
    const title = `posts[${postIndex}]`;
    await this.scene.startSection();

    const postContent = new PostContent({}).setTitle(title);
    for (const { postId, weight } of citations) {
      postContent.addCitation(postId, weight);
    }

    const { pool, postId } = await author.submitPostWithFee(
      this.bench,
      this.forum,
      postContent,
      {
        fee,
        duration: this.options.poolDurationMs,
        tokenLossRatio: 1,
      },
    );
    this.posts.push(postId);
    await delay(this.options.poolDurationMs);
    await pool.evaluateWinningConditions();
    await this.scene.endSection();
    await delay(this.options.defaultDelayMs);
    return postId;
  }

  async setup() {
    const rootElement = document.getElementById('scene');
    const rootBox = new Box('rootBox', rootElement).flex();

    this.scene = (window.scene = new Scene('Forum test', rootBox));
    this.scene.withSequenceDiagram();
    this.scene.withFlowchart();
    this.scene.withTable();

    this.scene.addDisplayValue('c3. stakeForAuthor').set(params.stakeForAuthor);
    this.scene.addDisplayValue('q2. revaluationLimit').set(params.revaluationLimit);
    this.scene
      .addDisplayValue('q3. referenceChainLimit')
      .set(params.referenceChainLimit);
    this.scene.addDisplayValue('q4. leachingValue').set(params.leachingValue);
    this.scene.addDisplayValue('&nbsp;');

    this.forum = (window.forum = new Forum('Forum', this.scene));
    this.bench = (window.bench = new Bench(this.forum, 'Bench', this.scene));
    this.experts = (window.experts = []);
    this.posts = (window.posts = []);

    await this.newExpert();
    // await newExpert();
    // await newExpert();

    // bench.addValue('total rep', () => bench.reputation.getTotal());
    this.forum.addValue('total value', () => this.forum.getTotalValue());
  }
}
